///////////////////////////////////////////////////////////////////////////////
// The SlideShowPanel class is a control for displaying content in a slide show. The panel
// can be hidden and made visible.

// Constructor.
function SlideShowPanel(id, caption, content, foldable, visible) {
    if (id != UI_NO_INIT_ID) {
        this.init(id, caption, content, foldable, visible);
    }
}

// SlideShowPanel inherits from Control.
SlideShowPanel.prototype = new Control(UI_NO_INIT_ID);

// The element hierarchy in a content panel is as follows:
//
// rootElement
//     assemblyElement
//         captionElement
//             foldToggleElement
//                     captionTextElement
//							subCaptionTextElement
//     contentElement
//
// captionTextElement is moved under foldToggleElement if disabled
// or captionElement if not foldable

// The fold toggle element used for folding content panels.
SlideShowPanel.prototype.foldToggleElement = null;

// The caption link element of this control.
SlideShowPanel.prototype.captionLinkElement = null;

// The caption text element of this control.
SlideShowPanel.prototype.captionTextElement = null;

// The sub-caption text element of this control.
SlideShowPanel.prototype.subCaptionTextElement = null;

// The play element
//SlideShowPanel.prototype.playElement = null;

// The stop element
//SlideShowPanel.prototype.stopElement = null;

// The content element of this control.
SlideShowPanel.prototype.contentElement = null;

// The foldable state of this control.
SlideShowPanel.prototype.foldable = false;

// The visible state of this control.
SlideShowPanel.prototype.visible = false;

// Enabled status.
SlideShowPanel.prototype.enabled = false;

// running status.
//SlideShowPanel.prototype.running = false;

// sets state of play stop controls
SlideShowPanel.prototype.setControlsState = function(running) {
}

SlideShowPanel.prototype.setRunning = function(state) {
	this.running=state;
	return;
}

// Initializer - called from constructor.
SlideShowPanel.prototype.init = function(id, caption, content, foldable, visible) {
    uiLogger.debug("SlideShowPanel.init(" + id + ", " + caption + ", " + content + ", " + foldable + ", " + visible + ")");
    
    // call superclass initializer
    Control.prototype.init.call(this, id, caption);
    
    // the control defaults to enabled
    this.enabled = true;
    
    // create caption text element
    this.captionTextElement = document.createElement("span");

    // disconnect the control element
    this.assemblyElement.removeChild(this.controlElement);
    
    // set the foldable state
    this.foldable = foldable;
     
    // is this a foldable content panel?
    if (foldable) {
        // create fold toggle element
        this.foldToggleElement = document.createElement("div");
        this.captionElement.appendChild(this.foldToggleElement);
        
        // create caption link and add to caption element
        this.foldToggleElement.appendChild(this.captionTextElement);
        

        this.subCaptionTextElement = document.createElement("div");
        this.foldToggleElement.appendChild(this.subCaptionTextElement);


        
        // bind event listeners
        var self = this;
        this.foldToggleElement.addEventListener("mouseover", function() { self.hoverStateChanged(true); }, false);
        this.foldToggleElement.addEventListener("mouseout", function() { self.hoverStateChanged(false); }, false);
        this.foldToggleElement.addEventListener("mousedown", function(event) {
                                                                 self.captionClicked();
                                                                 event.stopPropagation();
                                                                 event.preventDefault();
                                                             }, true);
        this.foldToggleElement.addEventListener("keydown", function(event) {
                                                               // center and enter trigger the action
                                                               if (event.keyCode == 0 || event.keyCode == 13) {
                                                                   self.captionClicked();
                                                                   event.stopPropagation();
                                                                   event.preventDefault();
                                                               }
                                                           }, true);
        
        this.visible = visible;
    } else {
        // since this is not a foldable panel the content should be visible
        this.visible = true;
        
        // add the text element directly to the caption element
        this.captionElement.appendChild(this.captionTextElement);
    }
    
    // create content element
    this.contentElement = document.createElement("div");
    this.contentElement.style.display = this.visible ? "block" : "none";
    this.rootElement.appendChild(this.contentElement);
    
    // set caption, content and visible state
    this.setCaption(caption);
    this.setContent(content);
    
    // update style
    this.updateStyleFromState();
}

// Returns the enabled state.
SlideShowPanel.prototype.isEnabled = function() {
    return this.enabled;
}

// Sets the enabled state.
SlideShowPanel.prototype.setEnabled = function(enabled) {
    uiLogger.debug("SlideShowPanel.setEnabled(" + enabled + ")");
    
    // bail out early if there is no change in state
    if (this.enabled == enabled) {
        return;
    }
    
    // set the enabled state
    this.enabled = enabled;
    
    // is this a foldable content?
    if (this.foldable) {
         // the caption link must be disabled
        if (this.enabled) {
            // diabled -> enabled
            this.foldToggleElement.removeChild(this.captionTextElement);
        } else {
            // enabled -> diabled
            this.foldToggleElement.appendChild(this.captionTextElement);
        }
    }
    
    // update style
    this.updateStyleFromState();    
}

// Returns the caption; null if none.
SlideShowPanel.prototype.getCaption = function() {
    return this.caption;
}

// Sets the caption; null if none.
SlideShowPanel.prototype.setCaption = function(imageThumb,caption,subCaption) {
    // bail out if the caption text element has not been created
    // this is to prevent the superclass init calling this before
    // we've initialized our custom caption
    if (this.captionTextElement == null)
        return;
    if (caption == null)
        caption="";
    
    uiLogger.debug("SlideShowPanel.setCaption(" + caption + ")");
    
    // set the display style
    this.captionElement.style.display = (caption == null) ? "none" : "block";
    
    // set the caption
    this.caption = "";//caption;
	var pos=0;
	var subCap="";//subCaption;
    this.captionTextElement.innerHTML = "";// "<img src='"+imageThumb+"' border='0' />"+caption;
    this.subCaptionTextElement.innerHTML = "";// subCap;
    
    // update style
    this.updateStyleFromState();
}

// Returns the content.
SlideShowPanel.prototype.getContent = function() {
    return this.contentElement.innerHTML;
}

// Sets the content.
SlideShowPanel.prototype.setContent = function(content) {
    uiLogger.debug("SlideShowPanel.setContent(" + content + ")");
    this.contentElement.innerHTML = (content == null) ? "" : content;
}

// Returns the focusable state for the control.
SlideShowPanel.prototype.isFocusable = function() {
    // a content panel is focusable if it's foldable and enabled
    return (this.foldable && this.enabled);
}

// Sets the focused state for the control.
// Note: This may not always succeed.
SlideShowPanel.prototype.setFocused = function(focused) {
}

// Returns the visible state.
SlideShowPanel.prototype.isVisible = function() {
    return this.visible;
}

// Sets the visible state.
SlideShowPanel.prototype.setVisible = function(visible){
	uiLogger.debug("SlideShowPanel.setVisible(" + visible + ")");

    this.visible = visible;
    if (this.visible) {
        // expand
		this.assemblyElement.style.display ="block";
        this.contentElement.style.display = "block";
		this.captionElement.style.display = "block";
    } else {
        // collapse
		this.assemblyElement.style.display ="none";
        this.contentElement.style.display = "none";
		this.captionElement.style.display = "none";
    }
/*    
	if (this.contentElement) {
		if (this.contentElement.parentNode) {
			if (this.contentElement.parentNode.scrollTop) {
				this.contentElement.parentNode.scrollTop = 0;
			}
		}
	}
*/
	
    // notify event listeners
    this.fireEvent(this.createEvent("VisibleStateChanged", this.visible));
    
    // update the style
    this.updateStyleFromState();
}


// Returns the absolute position (y) of the given element.
SlideShowPanel.prototype.getAbsoluteTop = function(element) {
    // traverse from element to root and add top-offset
    // for each element we find on the way
    var absTop = 0;
    while (element != null) {
        absTop += element.offsetTop;
        element = element.offsetParent;
    }
    return absTop;
}

// Callback for when the caption is clicked.
SlideShowPanel.prototype.captionClicked = function() {
    uiLogger.debug("SlideShowPanel.captionClicked()");

}

// Updates the style of the control to reflects the state of the control.
SlideShowPanel.prototype.updateStyleFromState = function() {
    uiLogger.debug("SlideShowPanel.updateStyleFromState()");

    // determine the state name
    var stateName = this.getStyleStateName();
    
    // set root element class name
    this.setClassName(this.rootElement, "Control");

    // set the control assembly class names
    this.setClassName(this.assemblyElement, "ControlAssembly ControlAssembly" + stateName);
    
    if (this.foldable) {
        // foldable content panel
        this.setClassName(this.captionElement, "SlideShowPanelCaptionFoldable");
        this.setClassName(this.foldToggleElement, "SlideShowPanelFoldToggle SlideShowPanelFoldToggle" + (this.visible ? "Visible" : "Collapsed"));
    } else {
        // non-folding
        this.setClassName(this.captionElement, "SlideShowPanelCaptionNonFoldable");
    }
    
    // set the content caption text class names
    this.setClassName(this.captionTextElement, "SlideShowPanelCaptionText SlideShowPanelCaptionText" + stateName);

    // set the content caption text class names
    this.setClassName(this.subCaptionTextElement, "SlideShowPanelSubCaptionText SlideShowPanelSubCaptionText" + stateName);
    
    // set the content element class names
    this.setClassName(this.contentElement, "SlideShowPanelContent");
}

